<?php

namespace Litecms\Courses\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Courses\Http\Requests\CourseRequest;
use Litecms\Courses\Interfaces\CourseRepositoryInterface;
use Litecms\Courses\Models\Course;

/**
 * Resource controller class for course.
 */
class CourseResourceController extends BaseController
{

    /**
     * Initialize course resource controller.
     *
     * @param type CourseRepositoryInterface $course
     *
     * @return null
     */
    public function __construct(CourseRepositoryInterface $course)
    {
        parent::__construct();
        $this->repository = $course;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Courses\Repositories\Criteria\CourseResourceCriteria::class);
    }

    /**
     * Display a list of course.
     *
     * @return Response
     */
    public function index(CourseRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Courses\Repositories\Presenter\CoursePresenter::class)
                ->$function();
        }

        $courses = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('courses::course.names'))
            ->view('courses::course.index', true)
            ->data(compact('courses', 'view'))
            ->output();
    }

    /**
     * Display course.
     *
     * @param Request $request
     * @param Model   $course
     *
     * @return Response
     */
    public function show(CourseRequest $request, Course $course)
    {

        if ($course->exists) {
            $view = 'courses::course.show';
        } else {
            $view = 'courses::course.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('courses::course.name'))
            ->data(compact('course'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new course.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CourseRequest $request)
    {

        $course = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('courses::course.name')) 
            ->view('courses::course.create', true) 
            ->data(compact('course'))
            ->output();
    }

    /**
     * Create new course.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CourseRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $course                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('courses::course.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('courses/course/' . $course->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/courses/course'))
                ->redirect();
        }

    }

    /**
     * Show course for editing.
     *
     * @param Request $request
     * @param Model   $course
     *
     * @return Response
     */
    public function edit(CourseRequest $request, Course $course)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('courses::course.name'))
            ->view('courses::course.edit', true)
            ->data(compact('course'))
            ->output();
    }

    /**
     * Update the course.
     *
     * @param Request $request
     * @param Model   $course
     *
     * @return Response
     */
    public function update(CourseRequest $request, Course $course)
    {
        try {
            $attributes = $request->all();

            $course->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('courses::course.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('courses/course/' . $course->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('courses/course/' . $course->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the course.
     *
     * @param Model   $course
     *
     * @return Response
     */
    public function destroy(CourseRequest $request, Course $course)
    {
        try {

            $course->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('courses::course.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('courses/course/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('courses/course/' . $course->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple course.
     *
     * @param Model   $course
     *
     * @return Response
     */
    public function delete(CourseRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('courses::course.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('courses/course'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/courses/course'))
                ->redirect();
        }

    }

    /**
     * Restore deleted courses.
     *
     * @param Model   $course
     *
     * @return Response
     */
    public function restore(CourseRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('courses::course.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/courses/course'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/courses/course/'))
                ->redirect();
        }

    }

}
